package net.projectmonkey.object.mapper.analysis.matching;

import net.projectmonkey.object.mapper.analysis.cache.TypePair;
import net.projectmonkey.object.mapper.analysis.result.PropertyMapping;
import net.projectmonkey.object.mapper.analysis.result.PropertyPath;

import java.util.List;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 *
 * Determines matches between a provided list of source and destination property paths.
 *
 * N.B. Implementations of MatchingStrategy should be referenced statically
 * or define equals in a way which means that separate instances will
 * be considered equal if the output from them will be the same.
 *
 * This is required since it forms part of the {@link net.projectmonkey.object.mapper.analysis.cache.MappingsCacheKey}.
 *
 * @author Andy Moody
 */
public interface MatchingStrategy
{

	/**
	 * Locate matching paths between the source and destination properties
	 * @param types - the type pair being matched against.
	 * @param sourcePaths - each element of which represents a property (and it's children) in the source type
	 * @param destinationPaths - each element of which represents a property (and it's children) in the destination type
	 * @return the resolved mappings
	 */
	List<PropertyMapping> resolveMatches(TypePair<?,?> types, List<PropertyPath> sourcePaths, List<PropertyPath> destinationPaths);

}
