package net.projectmonkey.object.mapper.analysis.resolver;

import net.projectmonkey.object.mapper.analysis.result.PropertyPath;

import java.util.List;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 * Resolves a representation of the object graph starting from the root type.
 *
 * N.B. Implementations of PropertyResolver should be referenced statically
 * or define equals in a way which means that separate instances will
 * be considered equal if the output from them will be the same.
 *
 * This is required since it forms part of the {@link net.projectmonkey.object.mapper.analysis.cache.MappingsCacheKey}.
 *
 * @author Andy Moody
 */
public interface PropertyResolver
{

	/**
	 * @param clazz
	 * @return a list of paths - each of which represents
	 *         the object graph below a property in the root type.
	 */
	List<PropertyPath> resolvePropertyPaths(Class<?> clazz);

}
