package net.projectmonkey.object.mapper.analysis.result;

import net.projectmonkey.object.mapper.util.ReflectionUtil;
import net.projectmonkey.object.mapper.util.TypeResolver;

import java.lang.reflect.Field;
import java.lang.reflect.Type;
import java.util.Map;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 * @author Andy Moody
 */
public class FieldBasedPropertyPathElement extends PropertyPathElement
{

	private Field field;
	private Class<?> type;

	public FieldBasedPropertyPathElement(final Class<?> containingType, final Field field)
	{
		this(containingType, field, calculateGroups(field.getType(), field));
	}

	public FieldBasedPropertyPathElement(final Class<?> containingType, final Field field, final Map<Class<?>, PropertyConfiguration> configurationsByGroup)
	{
		super(field.getName(), configurationsByGroup);
		this.field = field;
		this.type = TypeResolver.resolveClass(getGenericType(), containingType);
	}

	@Override
	public Class<?> getType()
	{
		return type;
	}

	@Override
	public Type getGenericType()
	{
		return field.getGenericType();
	}

	@Override
	public Object getValue(final Object source)
	{
		return ReflectionUtil.getValue(field, source);
	}

	@Override
	public void setValue(final Object value, final Object destination)
	{
		ReflectionUtil.setValue(field, value, destination);
	}

	@Override
	public boolean equals(final Object o)
	{
		if(this == o)
		{
			return true;
		}
		if(o == null || getClass() != o.getClass())
		{
			return false;
		}

		FieldBasedPropertyPathElement that = (FieldBasedPropertyPathElement) o;

		return field == null ? that.field == null : field.equals(that.field);
	}

	@Override
	public int hashCode()
	{
		return field != null ? field.hashCode() : 0;
	}

}
