package net.projectmonkey.object.mapper.analysis.result;

import net.projectmonkey.object.mapper.util.ReflectionUtil;
import net.projectmonkey.object.mapper.util.TypeResolver;

import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.Map;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 * @author Andy Moody
 */
public class JavaBeanBasedPropertyPathElement extends PropertyPathElement
{

	private Method accessorMethod;
	private Method mutatorMethod;
	private Class<?> type;


	public JavaBeanBasedPropertyPathElement(final Class<?> containingType, final String name, final Method accessorMethod, final Method mutatorMethod)
	{
		this(containingType, name, accessorMethod, mutatorMethod, calculateGroups(accessorMethod.getReturnType(), accessorMethod, mutatorMethod));
	}

	public JavaBeanBasedPropertyPathElement(final Class<?> containingType, final String name, final Method accessorMethod, final Method mutatorMethod, final Map<Class<?>, PropertyConfiguration> configurationsByGroup)
	{
		super(name, configurationsByGroup);
		this.accessorMethod = accessorMethod;
		this.mutatorMethod = mutatorMethod;
		this.type = TypeResolver.resolveClass(getGenericType(), containingType);
	}

	@Override
	public Class<?> getType()
	{
		return type;
	}

	@Override
	public Type getGenericType()
	{
		return accessorMethod.getGenericReturnType();
	}

	@Override
	public Object getValue(final Object source)
	{
		return ReflectionUtil.getValue(accessorMethod, source);
	}

	@Override
	public void setValue(final Object value, final Object destination)
	{
		ReflectionUtil.setValue(mutatorMethod, value, destination);
	}

	@Override
	public boolean equals(final Object o)
	{
		if(this == o)
		{
			return true;
		}
		if(o == null || getClass() != o.getClass())
		{
			return false;
		}

		JavaBeanBasedPropertyPathElement that = (JavaBeanBasedPropertyPathElement) o;

		boolean accessorEqual = accessorMethod == null ? that.accessorMethod == null : accessorMethod.equals(that.accessorMethod);
		boolean mutatorEqual = mutatorMethod == null ? that.mutatorMethod == null : mutatorMethod.equals(that.mutatorMethod);
		return accessorEqual && mutatorEqual;
	}

	@Override
	public int hashCode()
	{
		int result = accessorMethod != null ? accessorMethod.hashCode() : 0;
		result = 31 * result + (mutatorMethod != null ? mutatorMethod.hashCode() : 0);
		return result;
	}

}
