package net.projectmonkey.object.mapper.analysis.result;

import java.util.ArrayList;
import java.util.List;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 * @author Andy Moody
 */
public class PropertyMapping
{

	private final PropertyPath sourcePath;
	private final PropertyPath destinationPath;
	private final List<PropertyMapping> children;
	private final List<PropertyPath> sourceIntermediatePaths;
	private final List<PropertyPath> destinationIntermediatePaths;

	public PropertyMapping(final PropertyPath sourcePath, final PropertyPath destinationPath, final List<PropertyMapping> children)
	{
		this(sourcePath, destinationPath, children, new ArrayList<PropertyPath>(), new ArrayList<PropertyPath>());
	}

	public PropertyMapping(final PropertyPath sourcePath, final PropertyPath destinationPath, final List<PropertyMapping> children,
						   final List<PropertyPath> sourceIntermediatePaths, final List<PropertyPath> destinationIntermediatePaths)
	{
		this.sourcePath = sourcePath;
		this.destinationPath = destinationPath;
		this.children = children;
		this.sourceIntermediatePaths = sourceIntermediatePaths;
		this.destinationIntermediatePaths = destinationIntermediatePaths;
	}

	public PropertyPathElement getSourceProperty()
	{
		return sourcePath.getProperty();
	}

	public PropertyPathElement getDestinationProperty()
	{
		return destinationPath.getProperty();
	}

	public PropertyPath getSourcePath()
	{
		return sourcePath;
	}

	public PropertyPath getDestinationPath()
	{
		return destinationPath;
	}

	public List<PropertyMapping> getChildren()
	{
		return children;
	}

	public List<PropertyPath> getDestinationIntermediatePaths()
	{
		return destinationIntermediatePaths;
	}

	public List<PropertyPath> getSourceIntermediatePaths()
	{
		return sourceIntermediatePaths;
	}

	@Override
	public String toString()
	{
		return "PropertyMapping{" + sourcePath + " ----> " + destinationPath + '}';
	}

}
