package net.projectmonkey.object.mapper.construction.converter;


import net.projectmonkey.object.mapper.construction.PopulationContext;
import net.projectmonkey.object.mapper.construction.type.GenericTypeResolver;
import net.projectmonkey.object.mapper.context.ExecutionContext;

import java.util.Map;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 * Puts the source object directly into the destination if the destinationType
 * is assignable from the source object type.
 *
 * <p/>
 * Adapted from ModelMapper (modelmapper.org)
 *
 * @author Jonathan Halterman
 * @author Andy Moody
 */
public class AssignableTypeConverter implements Converter<Object, Object>
{
	public static final AssignableTypeConverter INSTANCE = new AssignableTypeConverter();

	@Override
	public Object convert(final PopulationContext<Object, Object> context)
	{
		return context.getSource();
	}

	@Override
	public boolean canConvert(final PopulationContext<?,?> context)
	{
		Object source = context.getSource();
		Class<?> destinationType = context.getDestinationType();
		boolean toReturn = false;
		if(source != null)
		{
			Class<? extends Object> sourceClass = source.getClass();
			boolean assignable = destinationType.isAssignableFrom(sourceClass);
			if(assignable)
			{
				toReturn = true;
				if(!(context.getDestinationGenericType() instanceof Class))
				{
					toReturn = false;
				}
				else if (getGenericTypeResolvers().containsKey(sourceClass))
				{
					GenericTypeResolver genericTypeResolver = getGenericTypeResolvers().get(sourceClass);
					toReturn = genericTypeResolver.getSourceTypeArguments(source).equals(genericTypeResolver.getDestinationTypeArguments(source));
				}
			}
		}
		return toReturn;
	}

	private Map<Class<?>, GenericTypeResolver> getGenericTypeResolvers()
	{
		return ExecutionContext.getConfiguration().getGenericTypeResolvers();
	}

}
