package net.projectmonkey.object.mapper.construction.converter;

import net.projectmonkey.object.mapper.construction.PopulationContext;
import net.projectmonkey.object.mapper.construction.RootPopulationContext;

import java.util.HashMap;
import java.util.Map;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 * Converts to {@link Boolean} instances.
 * <p/>
 * Adapted from ModelMapper (modelmapper.org)
 *
 * @author Jonathan Halterman
 * @author Andy Moody
 */
public class BooleanConverter implements CacheableConverter<Object, Boolean>
{

	public static final BooleanConverter INSTANCE = new BooleanConverter();
	private static final Map<String, Boolean> mappedValues;

	static
	{
		String[] TRUE_STRINGS = new String[]{"true", "yes", "y", "on", "1"};
		String[] FALSE_STRINGS = new String[]{"false", "no", "n", "off", "0"};
		mappedValues = new HashMap<String, Boolean>();
		for(String s : FALSE_STRINGS)
		{
			mappedValues.put(s, Boolean.FALSE);
		}
		for(String s : TRUE_STRINGS)
		{
			mappedValues.put(s, Boolean.TRUE);
		}
	}

	private BooleanConverter(){}


	@Override
	public Boolean convert(final PopulationContext<Object, Boolean> context)
	{
		Boolean toReturn = null;
		Object source = context.getSource();
		Class<? extends Boolean> destinationType = context.getDestinationType();
		if(source != null)
		{
			PopulationContext stringPopulationContext = new RootPopulationContext(source, String.class);
			String stringValue = StringConverter.INSTANCE.convert(stringPopulationContext).toLowerCase();
			if(mappedValues.containsKey(stringValue))
			{
				toReturn = mappedValues.get(stringValue);
			}
			else
			{
				throw new RuntimeException("Unable to convert "+stringValue+" to Boolean");
			}
		}
		else if(destinationType.equals(Boolean.TYPE))
		{
			toReturn = Boolean.FALSE;
		}
		return toReturn;
	}

	@Override
	public boolean canConvert(final PopulationContext<?,?> context)
	{
		Class<?> destinationType = context.getDestinationType();
		return destinationType == Boolean.class || destinationType == Boolean.TYPE;
	}

	@Override
	public Class<? extends Object>[] getApplicableSourceTypes()
	{
		return new Class[]{Boolean.class, Boolean.TYPE, String.class};
	}

	@Override
	public Class<? extends Boolean>[] getApplicableDestinationTypes()
	{
		return new Class[]{Boolean.class, Boolean.TYPE};
	}
}