package net.projectmonkey.object.mapper.construction.converter;

import net.projectmonkey.object.mapper.construction.PopulationContext;
import net.projectmonkey.object.mapper.util.Primitives;

import java.util.Calendar;
import java.util.GregorianCalendar;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 * @author Andy Moody
 */
public class CalendarToLongConverter implements CacheableConverter<Calendar, Long>
{

	public static final CalendarToLongConverter INSTANCE = new CalendarToLongConverter();

	private CalendarToLongConverter(){}

	@Override
	public Long convert(final PopulationContext<Calendar, Long> context)
	{
		Long toReturn = null;
		Calendar source = context.getSource();
		if(source != null)
		{
			toReturn = source.getTime().getTime();
		}

		return toReturn;
	}

	@Override
	public boolean canConvert(final PopulationContext<?,?> context)
	{
		return context.getSource() instanceof Calendar && Long.class.equals(Primitives.wrapperFor(context.getDestinationType()));
	}

	@Override
	public Class<? extends Calendar>[] getApplicableSourceTypes()
	{
		return new Class[]{Calendar.class, GregorianCalendar.class};
	}

	@Override
	public Class<? extends Long>[] getApplicableDestinationTypes()
	{
		return new Class[]{Long.class, Long.TYPE};
	}
}
