package net.projectmonkey.object.mapper.construction.postprocessor.resolver;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

import net.projectmonkey.object.mapper.construction.PopulationContext;
import net.projectmonkey.object.mapper.analysis.result.PropertyConfiguration;
import net.projectmonkey.object.mapper.construction.ConfigurationResolutionUtil;
import net.projectmonkey.object.mapper.construction.postprocessor.PostProcessor;
import net.projectmonkey.object.mapper.context.ConversionConfiguration;

import java.util.ArrayList;
import java.util.List;

import static net.projectmonkey.object.mapper.construction.ConfigurationResolutionUtil.ConfiguredTypeRetriever;

/**
 * @author Andy Moody
 */
public class StandardPostProcessorResolver implements PostProcessorResolver
{

	public static StandardPostProcessorResolver INSTANCE = new StandardPostProcessorResolver();

	private static final ConfigurationResolutionUtil util = ConfigurationResolutionUtil.INSTANCE;
	private static final PostProcessorRetriever retriever = new PostProcessorRetriever();


	private StandardPostProcessorResolver(){}

	@Override
	public PostProcessor<?, ?> getFirstAppropriatePostProcessor(final PopulationContext<?, ?> context)
	{
		return util.getFirstAppropriate(context, retriever);
	}

	private static class PostProcessorRetriever implements ConfiguredTypeRetriever<PostProcessor>
	{

		@Override
		public Class<? extends PostProcessor> retrieveType(final PropertyConfiguration configuration)
		{
			return configuration.getPostProcessor();
		}

		@Override
		public Class<PostProcessor> getTopLevelType()
		{
			return PostProcessor.class;
		}

		@Override
		public List<? extends PostProcessor> getConfigured(final ConversionConfiguration configuration, final PopulationContext<?, ?> context)
		{
			return new ArrayList<PostProcessor>(configuration.getPostProcessors());
		}

		@Override
		public boolean isApplicable(final PostProcessor item, final PopulationContext<?, ?> context)
		{
			return item.isApplicableFor(context);
		}
	}

}
