package net.projectmonkey.object.mapper.construction.rule;

import net.projectmonkey.object.mapper.construction.PopulationContext;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 * Provides a means for determining whether specific mappings
 * should be included in the population process.
 *
 * Note that rules operate as Vetoers - if any rule says that the context
 * should not be mapped then it won't be.
 *
 * @author Andy Moody
 */
public interface MappingRule<S, T>
{

	/**
	 * @param context - the mapping context for the property
	 * @return true if the property should be mapped, false otherwise.
	 */
	boolean isIncluded(PopulationContext<S, T> context);

	/**
	 * Method used to validate whether a Rule is applicable for the context.
	 * Note that if this evaluates to false when the rule has been specifically
	 * targeted to the property or type using {@link net.projectmonkey.object.mapper.annotations.group.Group }
	 * or {@link net.projectmonkey.object.mapper.annotations.group.Groups } an error is triggered.
	 *
	 * @param context - the mapping context for the property
	 * @return true if the rules should be applied when mapping the specified context
	 * from the source to the destination, false otherwise.
	 */
	boolean isApplicableFor(PopulationContext<?, ?> context);

}
