package net.projectmonkey.object.mapper.construction.type;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

import net.projectmonkey.object.mapper.construction.PopulationContext;
import net.projectmonkey.object.mapper.analysis.cache.TypePair;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Implementation of DestinationTypeProvider which stores source ---> destination type
 * mappings in a map.
 *
 * @author Andy Moody
 */
public class DefaultDestinationTypeProvider implements DestinationTypeProvider
{
	private Map<Class<?>, Class<?>> typeMap = new HashMap<Class<?>, Class<?>>();
	private List<TypePair<?,?>> typePairs = new ArrayList<TypePair<?, ?>>();

	public DefaultDestinationTypeProvider(){}

	@Override
	public Class<?> resolveDestinationType(final PopulationContext<?, ?> context)
	{
		Class<?> toReturn = context.getDestinationType();
		Class<?> sourceType = context.getSourceType();
		if(typeMap.containsKey(sourceType))
		{
			toReturn = typeMap.get(sourceType);
		}
		else
		{
			Object source = context.getSource();
			if(source != null)
			{
				Class<?> actualSourceType = source.getClass();
				if(typeMap.containsKey(actualSourceType))
				{
					toReturn = typeMap.get(actualSourceType);
				}
			}
		}

		return toReturn;
	}

	@Override
	public void addTypes(final TypePair<?, ?> types)
	{
		this.typePairs.add(types);
		this.typeMap.put(types.getSourceType(), types.getDestinationType());
	}

	@Override
	public List<TypePair<?, ?>> getAllTypes()
	{
		return typePairs;
	}

	@Override
	public boolean equals(final Object o)
	{
		if(this == o)
		{
			return true;
		}
		if(o == null || getClass() != o.getClass())
		{
			return false;
		}

		DefaultDestinationTypeProvider that = (DefaultDestinationTypeProvider) o;

		if(typeMap != null ? !typeMap.equals(that.typeMap) : that.typeMap != null)
		{
			return false;
		}

		return true;
	}

	@Override
	public int hashCode()
	{
		return typeMap != null ? typeMap.hashCode() : 0;
	}
}
