package net.projectmonkey.object.mapper.construction.type;

import java.util.List;

/**
 * Interface allowing generic type resolution for source and destination type arguments
 * from the source object.
 *
 * @author Andy Moody
 */
public interface GenericTypeResolver<T>
{

	/**
	 * Resolve the source type arguments based on the source object
	 * @param source
	 * @return A list of classes in the order the type variables are declared
	 */
	List<Class<?>> getSourceTypeArguments(T source);

	/**
	 * Resolve the destination type arguments based on the source objects
	 * @param source
	 * @return A list of classes in the order the type variables are declared
	 */
	List<Class<?>> getDestinationTypeArguments(T source);

	/**
	 * @return the types for which this type resolver is appropriate
	 * Note that type resolution is based on the actual source object, not the declared field type (unless no source is found)
	*/
	Class<? extends T>[] getApplicableSourceTypes();

}
