package net.projectmonkey.object.mapper.construction.type;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

import net.projectmonkey.object.mapper.util.MapUtil;

import java.util.*;

/**
 * Simple generic type resolver for some common map types.
 *
 * @author Andy Moody
 */
public class MapGenericTypeResolver implements GenericTypeResolver<Map<?,?>>
{
	private Class<?> destinationKeyType;
	private Class<?> destinationValueType;

	public MapGenericTypeResolver(final Class<?> destinationKeyType, final Class<?> destinationValueType)
	{
		this.destinationKeyType = destinationKeyType;
		this.destinationValueType = destinationValueType;
	}

	@Override
	public List<Class<?>> getSourceTypeArguments(final Map<?,?> source)
	{
		Class<?> keyType = Object.class;
		Class<?> valueType = Object.class;
		if(MapUtil.hasElements(source))
		{
			Map.Entry entry = (Map.Entry) source.entrySet().iterator().next();
			keyType = entry.getKey().getClass();
			valueType = entry.getValue().getClass();
		}
		return Arrays.<Class<?>>asList(keyType,  valueType);
	}

	@Override
	public List<Class<?>> getDestinationTypeArguments(final Map<?,?> source)
	{
		return Arrays.asList(destinationKeyType, destinationValueType);
	}

	@Override
	public Class[] getApplicableSourceTypes()
	{
		return new Class[]{Map.class, HashMap.class, LinkedHashMap.class, EnumMap.class};
	}

}
